@@ -1,13 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2022 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for DebugMailbox Debug probes support."""
 
 import logging
-from typing import Any, Dict, Type
+from typing import Dict, Type
 
 import colorama
 import prettytable
@@ -28,6 +28,8 @@ PROBES = {
 
 logger = logging.getLogger(__name__)
 
+colorama.init()
+
 
 class ProbeDescription:
     """NamedTuple for DAT record of debug probe description."""
@@ -51,7 +53,7 @@ class ProbeDescription:
         self.description = description
         self.probe = probe
 
-    def get_probe(self, user_params: Dict = None) -> Any:
+    def get_probe(self, user_params: Dict = None) -> DebugProbe:
         """Get instance of probe.
 
         :param user_params: The dictionary with optional user parameters
@@ -70,7 +72,7 @@ class DebugProbes(list):
         :raises SPSDKError: Invalid input types has been used.
         """
         if isinstance(item, ProbeDescription):
-            super(DebugProbes, self).append(item)
+            super().append(item)
         else:
             raise SPSDKError("The list accepts only ProbeDescription object")
 
@@ -82,7 +84,7 @@ class DebugProbes(list):
         :raises SPSDKError: Invalid input types has been used.
         """
         if isinstance(item, ProbeDescription):
-            super(DebugProbes, self).insert(index, item)
+            super().insert(index, item)
         else:
             raise SPSDKError("The list accepts only ProbeDescription object")
 
@@ -115,8 +117,6 @@ class DebugProbes(list):
 
     def print(self) -> None:
         """Prints the List of Probes to nice colored table."""
-        colorama.init()
-
         # Print all PyOCD probes and then Pemicro with local index
         table = prettytable.PrettyTable(["#", "Interface", "Id", "Description"])
         table.align = "l"
@@ -157,16 +157,16 @@ class DebugProbeUtils:
         :param interface: None to scan all interfaces, otherwise the selected interface is scanned only.
         :param hardware_id: None to list all probes, otherwise the the only probe with matching
         :param user_params: The dictionary with optional user parameters
-        hardware id is listed.
+            hardware id is listed.
         :return: list of probe_description's
         """
         probes = DebugProbes()
-        for probe_key in PROBES:
-            if (interface is None) or (interface.lower() == probe_key):
+        for key, probe in PROBES.items():
+            if (interface is None) or (interface.lower() == key):
                 try:
-                    probes.extend(PROBES[probe_key].get_connected_probes(hardware_id, user_params))
+                    probes.extend(probe.get_connected_probes(hardware_id, user_params))
                 except SPSDKDebugProbeError as exc:
-                    logger.warning(f"The {probe_key} debug probe support is not ready({str(exc)}).")
+                    logger.warning(f"The {key} debug probe support is not ready({str(exc)}).")
 
         return probes
 
