@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2022 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -10,13 +10,14 @@ import json
 import logging
 import os
 import struct
-from typing import Optional
+from typing import Any, Dict, List, Optional
 
 from spsdk import SPSDKError
+from spsdk.apps.utils import load_configuration
+from spsdk.image import TZ_SCH_FILE
 from spsdk.utils.easy_enum import Enum
-from spsdk.utils.misc import format_value
-
-from .misc import parse_int
+from spsdk.utils.misc import format_value, value_to_int
+from spsdk.utils.schema_validator import ConfigTemplate, ValidationSchemas
 
 logger = logging.getLogger(__name__)
 
@@ -67,6 +68,21 @@ class TrustZone:
             raw_data=raw_data,
         )
 
+    @classmethod
+    def from_config(cls, config_data: Dict[str, Any]) -> "TrustZone":
+        """Alternate constructor using configuration data.
+
+        :raises SPSDKError: Invalid configuration file.
+        :return: TrustZone class instance.
+        """
+        try:
+            family = config_data["family"]
+            revision = config_data.get("revision")
+            presets = config_data["trustZonePreset"]
+            return cls.custom(family=family, customizations=presets, revision=revision)
+        except (TypeError, SPSDKError) as exc:
+            raise SPSDKError("Invalid TrustZone configuration file.") from exc
+
     def __init__(
         self,
         tz_type: TrustZoneType = TrustZoneType.ENABLED,
@@ -78,7 +94,7 @@ class TrustZone:
         """Initialize the trustzone."""
         self.type = tz_type
         self.family: Optional[str] = family
-        self.config: dict = self._load_config_file()
+        self.config: dict = self.load_config_file()
         self.customs: Optional[dict] = customizations
         self.revision: Optional[str] = revision
 
@@ -110,17 +126,97 @@ class TrustZone:
             if not TrustZone.validate_custom_data(self.presets, self.customs):
                 raise SPSDKError(
                     "Invalid register found in customization data:\n"
-                    "%s" % [item for item in self.customs if item not in self.presets]
+                    f"{[item for item in self.customs if item not in self.presets]}"
                 )
 
+    @classmethod
+    def get_validation_schemas_family(cls) -> List[Dict[str, Any]]:
+        """Create the validation schema just for supported families.
+
+        :return: List of validation schemas for TZ supported families.
+        """
+        sch_cfg = ValidationSchemas.get_schema_file(TZ_SCH_FILE)
+        return [sch_cfg["tz_family_rev"]]
+
+    @classmethod
+    def get_validation_schemas(cls, family: str, revision: str = "latest") -> List[Dict[str, Any]]:
+        """Create the validation schema.
+
+        :param family: Family description.
+        :param revision: Chip revision specification, as default, latest is used.
+        :raises SPSDKError: Family or revision is not supported.
+        :return: List of validation schemas.
+        """
+        config_file = cls.load_config_file()
+        sch_cfg = ValidationSchemas.get_schema_file(TZ_SCH_FILE)
+        preset_properties = {}
+
+        try:
+            real_rev = config_file[family][revision]
+
+            presets = load_configuration(
+                os.path.join(TrustZone.PRESET_DIR, config_file[family]["revisions"][real_rev])
+            )
+            for key, value in presets.items():
+                preset_properties[key] = {
+                    "type": ["string", "number"],
+                    "title": "TZ Preset",
+                    "description": f"Preset for {key}",
+                    "format": "number",
+                    "template_value": f"{value}",
+                }
+            sch_cfg["tz"]["properties"]["trustZonePreset"].pop("patternProperties")
+            sch_cfg["tz"]["properties"]["trustZonePreset"]["properties"] = preset_properties
+
+            return [sch_cfg["tz_family_rev"], sch_cfg["tz"]]
+        except (KeyError, SPSDKError) as exc:
+            raise SPSDKError(f"Family {family} or {revision} is not supported") from exc
+
+    @classmethod
+    def generate_config_template(cls, family: str) -> Dict[str, str]:
+        """Generate configuration for selected family.
+
+        :param family: Family description.
+        :return: Dictionary of individual templates (key is name of template, value is template itself).
+        """
+        ret: Dict[str, str] = {}
+
+        if family in cls.get_supported_families():
+            config_file = cls.load_config_file()
+            schemas = cls.get_validation_schemas(family)
+            override = {}
+            override["family"] = family
+            override["revisions"] = config_file[family]["latest"]
+
+            yaml_data = ConfigTemplate(
+                f"Trust Zone Configuration template for {family}.",
+                schemas,
+                override,
+            ).export_to_yaml()
+            ret[f"{family}_tz"] = yaml_data
+
+        return ret
+
     def __str__(self) -> str:
         return f"<TrustZone: type: {self.type} ({TrustZoneType.desc(self.type)})"
 
-    def _load_config_file(self) -> dict:
+    @classmethod
+    def load_config_file(cls) -> dict:
         """Load data from TZ config file."""
-        with open(self.CONFIG_FILE) as f:
+        with open(cls.CONFIG_FILE) as f:
             return json.load(f)
 
+    @staticmethod
+    def get_supported_families() -> List[str]:
+        """Return list of supported families.
+
+        :return: List of supported families.
+        """
+        tz_sch_cfg = ValidationSchemas().get_schema_file(TZ_SCH_FILE)
+        tz_families = tz_sch_cfg["tz_family_rev"]
+
+        return tz_families["properties"]["family"]["enum"]
+
     def get_families(self) -> list:
         """Return list of supported chip families."""
         return list(self.config.keys())
@@ -155,7 +251,8 @@ class TrustZone:
         """Parse raw data into 'customizations' format."""
         if len(self.presets) != len(raw_data) // 4:
             raise SPSDKError(
-                f"Incorrect raw_data length\nExpected: {len(self.presets)}, Got: {len(raw_data) // 4}"
+                "Trustzone binary file has incorrect raw_data length\n"
+                f"Expected: {len(self.presets)}, Got: {len(raw_data) // 4}"
             )
 
         registers = struct.unpack(f"<{len(raw_data) // 4}L", raw_data)
@@ -178,7 +275,7 @@ class TrustZone:
         logger.debug(self.customs)
         data = self.presets
         data.update(self.customs)
-        registers = [parse_int(item) for item in data.values()]
+        registers = [value_to_int(item) for item in data.values()]
         # transform data into binary format (little endian, 32b per register)
         return struct.pack(f"<{len(registers)}I", *registers)
 
