@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright (c) 2019-2021 NXP
+# Copyright (c) 2019-2022 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Module for functionality shared across all MBoot interfaces."""
 
 from abc import ABC
-from typing import Any, Union
+from typing import Union
 
-from spsdk.mboot.commands import CmdResponse
+from spsdk.mboot.commands import CmdPacket, CmdResponse
 
 
-class Interface(ABC):
+class MBootInterface(ABC):
     """Base class for all Mboot Interface classes."""
 
     @property
@@ -42,8 +42,12 @@ class Interface(ABC):
     def read(self) -> Union[CmdResponse, bytes]:
         """Read data from the device."""
 
-    def write(self, packet: Any) -> None:
+    def write(self, packet: Union[CmdPacket, bytes]) -> None:
         """Write a packet to the device."""
 
     def info(self) -> str:
         """Return string containing information about the interface."""
+
+
+# for backwards compatibility
+Interface = MBootInterface
